////////////////////////////////////////////////////////////////////////////////
//
// Util.cpp
//
// Utility functions used within the CdoHelper project.
//
////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "comdef.h"						// for _bstr_t
#include "assert.h"
#include "cdo.h"						// for our CDO wrapper classes
#include "Util.h"

////////////////////////////////////////////////////////////////////////////////
//
// SafeRelease
//     Safely calls Release() through the given IUnknown pointer.
//
// Syntax
//     void SafeRelease(IUnknown **ppUnk) throw();
//
// Parameters
//     ppUnk
//         Pointer to an IUnknown pointer of a COM object that is to be
//         released.
//
// Remarks
//     The SafeRelease function calls Release() through the given IUnknown
//     pointer, and sets the pointer to NULL. If the supplied pointer is
//     already NULL, SafeRelease() simply returns. If there is an error
//     while calling Release(), the pointer is set to NULL and the
//     function returns successfully.
//
////////////////////////////////////////////////////////////////////////////////

void SafeRelease(IUnknown **ppUnk) throw()
{
	try
	{
		(*ppUnk)->Release();
	}
	catch(...)
	{
	}
	try
	{
		*ppUnk = NULL;
	}
	catch(...)
	{
	}

} // SafeRelease

////////////////////////////////////////////////////////////////////////////////
//
// SetAutomationError
//     Creates an error info object for the current thread.
//     This version uses string parameters.
//
// Syntax
//     HRESULT SetAutomationError(
//         REFGUID rguid,
//         LPCOLESTR szSource,
//         LPCOLESTR szDescription,
//         LPCOLESTR szHelpFile,
//         DWORD dwHelpContext
//     ) throw();
//
// Parameters
//     rguid
//         The IID of the interface that defines the error. This enables
//         the code that handles the error to interpret errors based on
//         the interface involved, regardless of which object implements
//         the interface.
//     szSource
//         The ProgID of the class or application that is raising the error.
//     szDescription
//         A textual description of the error.
//     szHelpFile
//         The path of the help file that describes the error.
//     dwHelpContext
//         The help context ID for the error.
//
// Return Values
//     If the function is successful, S_OK is returned. If not, E_FAIL is
//     returned.
//
// Remarks
//     The SetAutomationError function creates and sets a generic error info
//     object for the current thread. The error info object allows Automation
//     clients to retrieve extended error information.
//
////////////////////////////////////////////////////////////////////////////////

HRESULT SetAutomationError(
	REFGUID rguid,
	LPCOLESTR szSource,
	LPCOLESTR szDescription,
	LPCOLESTR szHelpFile,
	DWORD dwHelpContext
) throw()
{
	ICreateErrorInfo *pCreateErrorInfo;
	IErrorInfo *pErrorInfo;

	try
	{
		HRX hrx;

		// Create the CreateErrorInfo object and set its values.
		hrx = CreateErrorInfo(&pCreateErrorInfo);
		assert(pCreateErrorInfo);
		hrx = pCreateErrorInfo->SetGUID(rguid);
		hrx = pCreateErrorInfo->SetSource(const_cast<LPOLESTR>(szSource));
		hrx = pCreateErrorInfo->SetDescription(
			const_cast<LPOLESTR>(szDescription));
		hrx = pCreateErrorInfo->SetHelpFile(const_cast<LPOLESTR>(szHelpFile));
		hrx = pCreateErrorInfo->SetHelpContext(dwHelpContext);

		// Cast the CreateErrorInfo object to an ErrorInfo object, and
		// associate it with the current thread.
		hrx = pCreateErrorInfo->QueryInterface(IID_IErrorInfo,
			(void **)&pErrorInfo);
		assert(pErrorInfo);
		hrx = SetErrorInfo(0, pErrorInfo);

		// All done.
		SafeRelease((IUnknown **)&pErrorInfo);
		SafeRelease((IUnknown **)&pCreateErrorInfo);
		return S_OK;

	}
	catch(...)
	{
		SafeRelease((IUnknown **)&pErrorInfo);
		SafeRelease((IUnknown **)&pCreateErrorInfo);
		return E_FAIL;

	}

} // SetAutomationError

////////////////////////////////////////////////////////////////////////////////
//
// SetAutomationError
//     Creates an error info object for the current thread.
//     This version uses string resource ID parameters.
//
// Syntax
//     HRESULT SetAutomationError(
//         REFGUID rguid,
//         UINT nSourceID,
//         UINT nDescriptionID,
//         UINT nHelpFileID,
//         DWORD dwHelpContext
//     ) throw();
//
// Parameters
//     rguid
//         The IID of the interface that defines the error. This enables
//         the code that handles the error to interpret errors based on
//         the interface involved, regardless of which object implements
//         the interface.
//     nSource
//         The string resource ID for the ProgID of the class or application
//         that is raising the error.
//     nDescription
//         The string resource ID for a textual description of the error.
//     nHelpFile
//         The string resource ID for the path of the help file that describes
//         the error.
//     dwHelpContext
//         The help context ID for the error.
//
// Return Values
//     If the function is successful, S_OK is returned. If not, E_FAIL is
//     returned.
//
// Remarks
//     The SetAutomationError function creates and sets a generic error info
//     object for the current thread. The error info object allows Automation
//     clients to retrieve extended error information.
//
////////////////////////////////////////////////////////////////////////////////

HRESULT SetAutomationError(
	REFGUID rguid,
	UINT nSourceID,
	UINT nDescriptionID,
	UINT nHelpFileID,
	DWORD dwHelpContext
) throw()
{
	try
	{
		// Return value.
		HRESULT hrReturn;

		// We need CString objects in order to load the strings from the
		// resource file.
		CString csSource;
		CString csDescription;
		CString csHelpFile;

		// We need BSTR objects to pass to the string version of
		// SetAutomationError.
		BSTR bstrSource;
		BSTR bstrDescription;
		BSTR bstrHelpFile;

		// Load the requested strings from the resource file.
		csSource.LoadString(nSourceID);
		csDescription.LoadString(nDescriptionID);
		csHelpFile.LoadString(nHelpFileID);

		// Convert the strings into BSTRs.
		bstrSource = csSource.AllocSysString();
		bstrDescription = csDescription.AllocSysString();
		bstrHelpFile = csHelpFile.AllocSysString();

		// Do the work.
		hrReturn = SetAutomationError(rguid, bstrSource, bstrDescription,
			bstrHelpFile, dwHelpContext);

		// Deallocate the BSTRs (not done by the callee in this case, because
		// the callee thinks the strings are OLESTRs, not BSTRs. That's ok, it
		// just means that we have to deallocate the BSTRs here).
		SysFreeString(bstrSource);
		SysFreeString(bstrDescription);
		SysFreeString(bstrHelpFile);

		// All done.
		return hrReturn;

	}
	catch(...)
	{
		return E_FAIL;

	}

} // SetAutomationError

////////////////////////////////////////////////////////////////////////////////
//
// Stream2BSTR
//     Reads text data from a stream and returns it in a BSTR.
//
// Syntax
//     BSTR Stream2BSTR(IStream *pStream);
//
// Parameters
//     pStream
//         The IStream to read.
//
// Return Values
//     A BSTR containing the data read from the IStream.
//
// Remarks
//     Prior to calling Stream2BSTR, position the IStream to the location
//     where reading should begin. The entire remainder of the stream is
//     read into the BSTR, and a terminating null is appended to the BSTR.
//
////////////////////////////////////////////////////////////////////////////////

BSTR Stream2BSTR(IStream *pStream)
{
	USES_CONVERSION;

	const ULONG cbBLOCK_SIZE = 256;
	ULONG cbBuffSize = 0;
	ULONG nCharCount = 0;
	char *pby = NULL;
	char *pbyNew = NULL;
	char *pbyCurrentBlock = NULL;
	ULONG cbBytesRead;
	HRX hrx;
	BSTR bstrRetVal;

	do
	{
		// We looped again, so we need a bigger buffer.
		cbBuffSize += cbBLOCK_SIZE;

		// Expand the buffer.
		pbyNew = (char *)realloc(pby, cbBuffSize);
		if (NULL == pbyNew)
		{
			free(pby);
			AfxThrowMemoryException();
		}
		pby = pbyNew;
		pbyNew = NULL;

		// We need a pointer into the current block.
		pbyCurrentBlock = pby + cbBuffSize - cbBLOCK_SIZE;
		assert((pbyCurrentBlock >= pby)
			&& (pbyCurrentBlock + cbBLOCK_SIZE == pby + cbBuffSize));

		// Read the current block.
		hrx = pStream->Read(pbyCurrentBlock, cbBLOCK_SIZE, &cbBytesRead);
		assert((pbyCurrentBlock + cbBytesRead) <= (pby + cbBuffSize));

	} while (cbBLOCK_SIZE == cbBytesRead);

	// Add a terminating null.
	*((OLECHAR *)(pbyCurrentBlock + cbBytesRead)) = 0;

	// Convert the ANSI string into a BSTR.
	bstrRetVal = A2BSTR(pby);

	// Free the buffer.
	free(pby);

	// All done.
	return bstrRetVal;

} // Stream2BSTR

////////////////////////////////////////////////////////////////////////////////
//
// BSTR2Stream
//     Converts a BSTR into an ANSI string and writes it to a stream.
//
// Syntax
//     HRESULT BSTR2Stream(
//         BSTR bstr,
//         IStream *pStream
//     );
//
// Parameters
//     bstr
//         The string to be written.
//     pStream
//         The stream to write to.
//
// Return Values
//     S_OK if the function is successful, or a failure value otherwise.
//
// Remarks
//     The stream should be opened for writing and positioned correctly
//     prior to calling BSTR2Stream.
//
///////////////////////////////////////////////////////////////////////////////

HRESULT BSTR2Stream(
	BSTR bstr,
	IStream *pStream
)
{
	USES_CONVERSION;

	HRESULT hr;

	// Convert the BSTR to an ANSI string. (Isn't ATL marvelous?)
	char *sz = W2A(bstr);

	ULONG cbLen = strlen(sz);
	ULONG cbWritten = 0;

	// Write the ANSI version of the string to the stream.
	hr = pStream->Write(sz, cbLen, &cbWritten);
	if (FAILED(hr))
		return hr;

	return S_OK;

} // BSTR2Stream
