////////////////////////////////////////////////////////////////////////////////
//
// Util.h
//
// Header file for Util.cpp
//
////////////////////////////////////////////////////////////////////////////////

#ifndef __UTIL_H__
#define __UTIL_H__

#include "cdo.h"						// for our CDO wrapper classes

// Disable compiler warning 4290 ("C++ Exception Specification ignored").
// Visual C++ 6.0 doesn't implement exception specification checking, but it
// allows exception specifications to exist. When exception specifications
// are seen by the compiler, it generates this warning to alert the developer.
// Disabling this warning allows us to use exception specifications to aid
// in clarity, while avoiding the warning output.
#pragma warning(disable:4290)

////////////////////////////////////////////////////////////////////////////////
//
// Class HRX
//
// The HRX class turns failed HRESULTS into C++ exceptions. HRX objects can
// be used in most contexts in which HRESULTs are used. The difference is that
// if an HRX object is assigned or constructed with an HRESULT value that
// indicates error, then an exception (of type HRESULT) is thrown. This relieves
// the main code from having to continually check and act on HRESULTs. Instead,
// error handling code can be collected into a common catch block. For example:
//
//     try
//     {
//         HRX hrx = SomeCOMCall();
//         hrx = SomeOtherCOMCall();
//         HRX(StillAnotherCOMCall());
//     }
//     catch(HRESULT hr)
//     {
//         // take some appropriate action
//     }
//
// HRX allows the compiler to generate default address-of operators, which
// return HRX pointers, not HRESULT pointers. This is the only context in
// which an HRX object can't be used in place of an HRESULT. The reason the
// HRX class doesn't define address-of operators that return pointers to
// HRESULTs is that it would then be impossible to intercept assignment to
// the pointed to address.
//
// The idea for an HRESULT wrapper is from the book, "Effective COM,"
// by Don Box, Keith Brown, Tim Ewald, and Chris Sells.
//
////////////////////////////////////////////////////////////////////////////////

class HRX
{
protected:
	HRESULT m_hr;						// holds the HRESULT value

	void assign(HRESULT hr)				// assigns hr to this object
	throw(HRESULT)
	{
		m_hr = hr;
		if (FAILED(hr))
			throw(hr);
	}

public:
	HRX()								// default constructor
	{ assign(NULL); }
	HRX(const HRESULT hr)				// construct from HRESULT
	throw(HRESULT) { assign(hr); }
	~HRX() {}							// destructor
	HRX& operator=(const HRESULT hr)	// assignment operator
	throw(HRESULT) { assign(hr); return *this; }
	operator HRESULT() const			// conversion to HRESULT
	{ return m_hr; }

}; // class HRX

// Safely calls Release() through the given IUnknown pointer.
void SafeRelease(IUnknown **ppUnk) throw();

// Creates an error info object for the current thread.
// This version uses string parameters.
HRESULT SetAutomationError(
	REFGUID rguid,
	LPCOLESTR szSource,
	LPCOLESTR szDescription,
	LPCOLESTR szHelpFile,
	DWORD dwHelpContext
) throw();

// Creates an error info object for the current thread.
// This version uses string resource ID parameters.
HRESULT SetAutomationError(
	REFGUID rguid,
	UINT nSourceID,
	UINT nDescriptionID,
	UINT nHelpFileID,
	DWORD dwHelpContext
) throw();

// Converts a hex character to its byte value.
inline BYTE HexChar2Val(OLECHAR c);

// Converts the BSTR representation of compressed RTF data into an
// IStream suitable for use by WrapCompressedRTFStream().
HRESULT HexBSTR2Stream(
	BSTR bstr,
	IStream **ppStream
);

// Reads text data from a stream and returns it in a BSTR.
BSTR Stream2BSTR(IStream *pStream);

// Returns the value of the given property on the given message.
VARIANT GetMessagePropertyValue(
	CCdoMessage &CdoMessage,
	ULONG ulPropertyID
);

// Converts a hex value to its character equivalent.
OLECHAR Val2HexChar(BYTE by);

// Converts a stream of compressed RTF to the BSTR representation used
// by CDO.
HRESULT Stream2HexBSTR(
	IStream *pStream,
	BSTR *pbstr
);

// Converts a BSTR into an ANSI string and writes it to a stream.
HRESULT BSTR2Stream(
	BSTR bstr,
	IStream *pStream
);

#endif // __UTIL_H__